@@ -69,7 +69,7 @@ void connchk_timeout_cb(struct ev_loop *loop, ev_timer *w, int revents) {
   ev_timer_stop(loop, w);
 
   switch (http2session->get_connection_check_state()) {
-  case Http2Session::CONNECTION_CHECK_STARTED:
+  case ConnectionCheck::STARTED:
     // ping timeout; disconnect
     if (LOG_ENABLED(INFO)) {
       SSLOG(INFO, http2session) << "ping timeout";
@@ -82,8 +82,7 @@ void connchk_timeout_cb(struct ev_loop *loop, ev_timer *w, int revents) {
     if (LOG_ENABLED(INFO)) {
       SSLOG(INFO, http2session) << "connection check required";
     }
-    http2session->set_connection_check_state(
-        Http2Session::CONNECTION_CHECK_REQUIRED);
+    http2session->set_connection_check_state(ConnectionCheck::REQUIRED);
   }
 }
 } // namespace
@@ -189,7 +188,7 @@ Http2Session::Http2Session(struct ev_loop *loop, SSL_CTX *ssl_ctx,
             group->shared_addr->timeout.write, group->shared_addr->timeout.read,
             {}, {}, writecb, readcb, timeoutcb, this,
             get_config()->tls.dyn_rec.warmup_threshold,
-            get_config()->tls.dyn_rec.idle_timeout, PROTO_HTTP2),
+            get_config()->tls.dyn_rec.idle_timeout, Proto::HTTP2),
       wb_(worker->get_mcpool()),
       worker_(worker),
       ssl_ctx_(ssl_ctx),
@@ -197,9 +196,9 @@ Http2Session::Http2Session(struct ev_loop *loop, SSL_CTX *ssl_ctx,
       addr_(addr),
       session_(nullptr),
       raddr_(nullptr),
-      state_(DISCONNECTED),
-      connection_check_state_(CONNECTION_CHECK_NONE),
-      freelist_zone_(FREELIST_ZONE_NONE),
+      state_(Http2SessionState::DISCONNECTED),
+      connection_check_state_(ConnectionCheck::NONE),
+      freelist_zone_(FreelistZone::NONE),
       settings_recved_(false),
       allow_connect_proto_(false) {
   read_ = write_ = &Http2Session::noop;
@@ -266,8 +265,8 @@ int Http2Session::disconnect(bool hard) {
     proxy_htp_.reset();
   }
 
-  connection_check_state_ = CONNECTION_CHECK_NONE;
-  state_ = DISCONNECTED;
+  connection_check_state_ = ConnectionCheck::NONE;
+  state_ = Http2SessionState::DISCONNECTED;
 
   // When deleting Http2DownstreamConnection, it calls this object's
   // remove_downstream_connection().  The multiple
@@ -305,13 +304,11 @@ int Http2Session::disconnect(bool hard) {
 }
 
 int Http2Session::resolve_name() {
-  int rv;
-
-  auto dns_query = make_unique<DNSQuery>(
-      addr_->host, [this](int status, const Address *result) {
+  auto dns_query = std::make_unique<DNSQuery>(
+      addr_->host, [this](DNSResolverStatus status, const Address *result) {
         int rv;
 
-        if (status == DNS_STATUS_OK) {
+        if (status == DNSResolverStatus::OK) {
           *resolved_addr_ = *result;
           util::set_port(*this->resolved_addr_, this->addr_->port);
         }
@@ -321,17 +318,16 @@ int Http2Session::resolve_name() {
           delete this;
         }
       });
-  resolved_addr_ = make_unique<Address>();
+  resolved_addr_ = std::make_unique<Address>();
   auto dns_tracker = worker_->get_dns_tracker();
-  rv = dns_tracker->resolve(resolved_addr_.get(), dns_query.get());
-  switch (rv) {
-  case DNS_STATUS_ERROR:
+  switch (dns_tracker->resolve(resolved_addr_.get(), dns_query.get())) {
+  case DNSResolverStatus::ERROR:
     return -1;
-  case DNS_STATUS_RUNNING:
+  case DNSResolverStatus::RUNNING:
     dns_query_ = std::move(dns_query);
-    state_ = RESOLVING_NAME;
+    state_ = Http2SessionState::RESOLVING_NAME;
     return 0;
-  case DNS_STATUS_OK:
+  case DNSResolverStatus::OK:
     util::set_port(*resolved_addr_, addr_->port);
     return 0;
   default:
@@ -345,7 +341,8 @@ int Http2Session::initiate_connection() {
 
   auto worker_blocker = worker_->get_connect_blocker();
 
-  if (state_ == DISCONNECTED || state_ == RESOLVING_NAME) {
+  if (state_ == Http2SessionState::DISCONNECTED ||
+      state_ == Http2SessionState::RESOLVING_NAME) {
     if (worker_blocker->blocked()) {
       if (LOG_ENABLED(INFO)) {
         SSLOG(INFO, this)
@@ -358,7 +355,7 @@ int Http2Session::initiate_connection() {
   auto &downstreamconf = *get_config()->conn.downstream;
 
   const auto &proxy = get_config()->downstream_http_proxy;
-  if (!proxy.host.empty() && state_ == DISCONNECTED) {
+  if (!proxy.host.empty() && state_ == Http2SessionState::DISCONNECTED) {
     if (LOG_ENABLED(INFO)) {
       SSLOG(INFO, this) << "Connecting to the proxy " << proxy.host << ":"
                         << proxy.port;
@@ -405,26 +402,27 @@ int Http2Session::initiate_connection() {
     on_read_ = &Http2Session::downstream_read_proxy;
     on_write_ = &Http2Session::downstream_connect_proxy;
 
-    proxy_htp_ = make_unique<http_parser>();
+    proxy_htp_ = std::make_unique<http_parser>();
     http_parser_init(proxy_htp_.get(), HTTP_RESPONSE);
     proxy_htp_->data = this;
 
-    state_ = PROXY_CONNECTING;
+    state_ = Http2SessionState::PROXY_CONNECTING;
 
     return 0;
   }
 
-  if (state_ == DISCONNECTED || state_ == PROXY_CONNECTED ||
-      state_ == RESOLVING_NAME) {
+  if (state_ == Http2SessionState::DISCONNECTED ||
+      state_ == Http2SessionState::PROXY_CONNECTED ||
+      state_ == Http2SessionState::RESOLVING_NAME) {
     if (LOG_ENABLED(INFO)) {
-      if (state_ != RESOLVING_NAME) {
+      if (state_ != Http2SessionState::RESOLVING_NAME) {
         SSLOG(INFO, this) << "Connecting to downstream server";
       }
     }
     if (addr_->tls) {
       assert(ssl_ctx_);
 
-      if (state_ != RESOLVING_NAME) {
+      if (state_ != Http2SessionState::RESOLVING_NAME) {
         auto ssl = tls::create_ssl(ssl_ctx_);
         if (!ssl) {
           return -1;
@@ -452,14 +450,14 @@ int Http2Session::initiate_connection() {
         }
       }
 
-      if (state_ == DISCONNECTED) {
+      if (state_ == Http2SessionState::DISCONNECTED) {
         if (addr_->dns) {
           rv = resolve_name();
           if (rv != 0) {
             downstream_failure(addr_, nullptr);
             return -1;
           }
-          if (state_ == RESOLVING_NAME) {
+          if (state_ == Http2SessionState::RESOLVING_NAME) {
             return 0;
           }
           raddr_ = resolved_addr_.get();
@@ -468,20 +466,21 @@ int Http2Session::initiate_connection() {
         }
       }
 
-      if (state_ == RESOLVING_NAME) {
-        if (dns_query_->status == DNS_STATUS_ERROR) {
+      if (state_ == Http2SessionState::RESOLVING_NAME) {
+        if (dns_query_->status == DNSResolverStatus::ERROR) {
           downstream_failure(addr_, nullptr);
           return -1;
         }
-        assert(dns_query_->status == DNS_STATUS_OK);
-        state_ = DISCONNECTED;
+        assert(dns_query_->status == DNSResolverStatus::OK);
+        state_ = Http2SessionState::DISCONNECTED;
         dns_query_.reset();
         raddr_ = resolved_addr_.get();
       }
 
-      // If state_ == PROXY_CONNECTED, we has connected to the proxy
-      // using conn_.fd and tunnel has been established.
-      if (state_ == DISCONNECTED) {
+      // If state_ == Http2SessionState::PROXY_CONNECTED, we have
+      // connected to the proxy using conn_.fd and tunnel has been
+      // established.
+      if (state_ == Http2SessionState::DISCONNECTED) {
         assert(conn_.fd == -1);
 
         conn_.fd = util::create_nonblock_socket(raddr_->su.storage.ss_family);
@@ -516,7 +515,7 @@ int Http2Session::initiate_connection() {
 
       conn_.prepare_client_handshake();
     } else {
-      if (state_ == DISCONNECTED) {
+      if (state_ == Http2SessionState::DISCONNECTED) {
         // Without TLS and proxy.
         if (addr_->dns) {
           rv = resolve_name();
@@ -524,7 +523,7 @@ int Http2Session::initiate_connection() {
             downstream_failure(addr_, nullptr);
             return -1;
           }
-          if (state_ == RESOLVING_NAME) {
+          if (state_ == Http2SessionState::RESOLVING_NAME) {
             return 0;
           }
           raddr_ = resolved_addr_.get();
@@ -533,18 +532,18 @@ int Http2Session::initiate_connection() {
         }
       }
 
-      if (state_ == RESOLVING_NAME) {
-        if (dns_query_->status == DNS_STATUS_ERROR) {
+      if (state_ == Http2SessionState::RESOLVING_NAME) {
+        if (dns_query_->status == DNSResolverStatus::ERROR) {
           downstream_failure(addr_, nullptr);
           return -1;
         }
-        assert(dns_query_->status == DNS_STATUS_OK);
-        state_ = DISCONNECTED;
+        assert(dns_query_->status == DNSResolverStatus::OK);
+        state_ = Http2SessionState::DISCONNECTED;
         dns_query_.reset();
         raddr_ = resolved_addr_.get();
       }
 
-      if (state_ == DISCONNECTED) {
+      if (state_ == Http2SessionState::DISCONNECTED) {
         // Without TLS and proxy.
         assert(conn_.fd == -1);
 
@@ -580,7 +579,7 @@ int Http2Session::initiate_connection() {
     }
 
     // We have been already connected when no TLS and proxy is used.
-    if (state_ == PROXY_CONNECTED) {
+    if (state_ == Http2SessionState::PROXY_CONNECTED) {
       on_read_ = &Http2Session::read_noop;
       on_write_ = &Http2Session::write_noop;
 
@@ -589,7 +588,7 @@ int Http2Session::initiate_connection() {
 
     write_ = &Http2Session::connected;
 
-    state_ = CONNECTING;
+    state_ = Http2SessionState::CONNECTING;
     conn_.wlimit.startw();
 
     conn_.wt.repeat = downstreamconf.timeout.connect;
@@ -624,13 +623,13 @@ int htp_hdrs_completecb(http_parser *htp) {
     if (LOG_ENABLED(INFO)) {
       SSLOG(INFO, http2session) << "Tunneling success";
     }
-    http2session->set_state(Http2Session::PROXY_CONNECTED);
+    http2session->set_state(Http2SessionState::PROXY_CONNECTED);
 
     return 0;
   }
 
   SSLOG(WARN, http2session) << "Tunneling failed: " << htp->status_code;
-  http2session->set_state(Http2Session::PROXY_FAILED);
+  http2session->set_state(Http2SessionState::PROXY_FAILED);
 
   return 0;
 }
@@ -659,14 +658,16 @@ int Http2Session::downstream_read_proxy(const uint8_t *data, size_t datalen) {
 
   if (htperr == HPE_PAUSED) {
     switch (state_) {
-    case Http2Session::PROXY_CONNECTED:
+    case Http2SessionState::PROXY_CONNECTED:
       // Initiate SSL/TLS handshake through established tunnel.
       if (initiate_connection() != 0) {
         return -1;
       }
       return 0;
-    case Http2Session::PROXY_FAILED:
+    case Http2SessionState::PROXY_FAILED:
       return -1;
+    default:
+      break;
     }
     // should not be here
     assert(0);
@@ -726,7 +727,7 @@ void Http2Session::remove_downstream_connection(
     SSLOG(INFO, this) << "Remove downstream";
   }
 
-  if (freelist_zone_ == FREELIST_ZONE_NONE && !max_concurrency_reached()) {
+  if (freelist_zone_ == FreelistZone::NONE && !max_concurrency_reached()) {
     if (LOG_ENABLED(INFO)) {
       SSLOG(INFO, this) << "Append to http2_extra_freelist, addr=" << addr_
                         << ", freelist.size="
@@ -748,8 +749,8 @@ void Http2Session::remove_stream_data(StreamData *sd) {
 int Http2Session::submit_request(Http2DownstreamConnection *dconn,
                                  const nghttp2_nv *nva, size_t nvlen,
                                  const nghttp2_data_provider *data_prd) {
-  assert(state_ == CONNECTED);
-  auto sd = make_unique<StreamData>();
+  assert(state_ == Http2SessionState::CONNECTED);
+  auto sd = std::make_unique<StreamData>();
   sd->dlnext = sd->dlprev = nullptr;
   // TODO Specify nullptr to pri_spec for now
   auto stream_id =
@@ -768,7 +769,7 @@ int Http2Session::submit_request(Http2DownstreamConnection *dconn,
 }
 
 int Http2Session::submit_rst_stream(int32_t stream_id, uint32_t error_code) {
-  assert(state_ == CONNECTED);
+  assert(state_ == Http2SessionState::CONNECTED);
   if (LOG_ENABLED(INFO)) {
     SSLOG(INFO, this) << "RST_STREAM stream_id=" << stream_id
                       << " with error_code=" << error_code;
@@ -786,7 +787,7 @@ int Http2Session::submit_rst_stream(int32_t stream_id, uint32_t error_code) {
 nghttp2_session *Http2Session::get_session() const { return session_; }
 
 int Http2Session::resume_data(Http2DownstreamConnection *dconn) {
-  assert(state_ == CONNECTED);
+  assert(state_ == Http2SessionState::CONNECTED);
   auto downstream = dconn->get_downstream();
   int rv = nghttp2_session_resume_data(session_,
                                        downstream->get_downstream_stream_id());
@@ -836,34 +837,34 @@ int on_stream_close_callback(nghttp2_session *session, int32_t stream_id,
     auto upstream = downstream->get_upstream();
 
     if (downstream->get_downstream_stream_id() % 2 == 0 &&
-        downstream->get_request_state() == Downstream::INITIAL) {
+        downstream->get_request_state() == DownstreamState::INITIAL) {
       // Downstream is canceled in backend before it is submitted in
       // frontend session.
 
       // This will avoid to send RST_STREAM to backend
-      downstream->set_response_state(Downstream::MSG_RESET);
+      downstream->set_response_state(DownstreamState::MSG_RESET);
       upstream->cancel_premature_downstream(downstream);
     } else {
-      if (downstream->get_upgraded() &&
-          downstream->get_response_state() == Downstream::HEADER_COMPLETE) {
+      if (downstream->get_upgraded() && downstream->get_response_state() ==
+                                            DownstreamState::HEADER_COMPLETE) {
         // For tunneled connection, we have to submit RST_STREAM to
         // upstream *after* whole response body is sent. We just set
         // MSG_COMPLETE here. Upstream will take care of that.
         downstream->get_upstream()->on_downstream_body_complete(downstream);
-        downstream->set_response_state(Downstream::MSG_COMPLETE);
+        downstream->set_response_state(DownstreamState::MSG_COMPLETE);
       } else if (error_code == NGHTTP2_NO_ERROR) {
         switch (downstream->get_response_state()) {
-        case Downstream::MSG_COMPLETE:
-        case Downstream::MSG_BAD_HEADER:
+        case DownstreamState::MSG_COMPLETE:
+        case DownstreamState::MSG_BAD_HEADER:
           break;
         default:
-          downstream->set_response_state(Downstream::MSG_RESET);
+          downstream->set_response_state(DownstreamState::MSG_RESET);
         }
       } else if (downstream->get_response_state() !=
-                 Downstream::MSG_BAD_HEADER) {
-        downstream->set_response_state(Downstream::MSG_RESET);
+                 DownstreamState::MSG_BAD_HEADER) {
+        downstream->set_response_state(DownstreamState::MSG_RESET);
       }
-      if (downstream->get_response_state() == Downstream::MSG_RESET &&
+      if (downstream->get_response_state() == DownstreamState::MSG_RESET &&
           downstream->get_response_rst_stream_error_code() ==
               NGHTTP2_NO_ERROR) {
         downstream->set_response_rst_stream_error_code(error_code);
@@ -1136,13 +1137,13 @@ int on_response_headers(Http2Session *http2session, Downstream *downstream,
     if (rv != 0) {
       http2session->submit_rst_stream(frame->hd.stream_id,
                                       NGHTTP2_PROTOCOL_ERROR);
-      downstream->set_response_state(Downstream::MSG_RESET);
+      downstream->set_response_state(DownstreamState::MSG_RESET);
     }
 
     return 0;
   }
 
-  downstream->set_response_state(Downstream::HEADER_COMPLETE);
+  downstream->set_response_state(DownstreamState::HEADER_COMPLETE);
   downstream->check_upgrade_fulfilled_http2();
 
   if (downstream->get_upgraded()) {
@@ -1153,7 +1154,7 @@ int on_response_headers(Http2Session *http2session, Downstream *downstream,
       delete handler;
       return -1;
     }
-    downstream->set_request_state(Downstream::HEADER_COMPLETE);
+    downstream->set_request_state(DownstreamState::HEADER_COMPLETE);
     if (LOG_ENABLED(INFO)) {
       SSLOG(INFO, http2session)
           << "HTTP upgrade success. stream_id=" << frame->hd.stream_id;
@@ -1173,7 +1174,7 @@ int on_response_headers(Http2Session *http2session, Downstream *downstream,
         resp.connection_close = true;
       } else {
         // Otherwise, use chunked encoding to keep upstream connection
-        // open.  In HTTP2, we are supporsed not to receive
+        // open.  In HTTP2, we are supposed not to receive
         // transfer-encoding.
         resp.fs.add_header_token(StringRef::from_lit("transfer-encoding"),
                                  StringRef::from_lit("chunked"), false,
@@ -1196,12 +1197,12 @@ int on_response_headers(Http2Session *http2session, Downstream *downstream,
   if (rv != 0) {
     // Handling early return (in other words, response was hijacked by
     // mruby scripting).
-    if (downstream->get_response_state() == Downstream::MSG_COMPLETE) {
+    if (downstream->get_response_state() == DownstreamState::MSG_COMPLETE) {
       http2session->submit_rst_stream(frame->hd.stream_id, NGHTTP2_CANCEL);
     } else {
       http2session->submit_rst_stream(frame->hd.stream_id,
                                       NGHTTP2_INTERNAL_ERROR);
-      downstream->set_response_state(Downstream::MSG_RESET);
+      downstream->set_response_state(DownstreamState::MSG_RESET);
     }
   }
 
@@ -1228,20 +1229,21 @@ int on_frame_recv_callback(nghttp2_session *session, const nghttp2_frame *frame,
     if (rv != 0) {
       http2session->submit_rst_stream(frame->hd.stream_id,
                                       NGHTTP2_INTERNAL_ERROR);
-      downstream->set_response_state(Downstream::MSG_RESET);
+      downstream->set_response_state(DownstreamState::MSG_RESET);
 
     } else if (frame->hd.flags & NGHTTP2_FLAG_END_STREAM) {
 
       downstream->disable_downstream_rtimer();
 
-      if (downstream->get_response_state() == Downstream::HEADER_COMPLETE) {
+      if (downstream->get_response_state() ==
+          DownstreamState::HEADER_COMPLETE) {
 
-        downstream->set_response_state(Downstream::MSG_COMPLETE);
+        downstream->set_response_state(DownstreamState::MSG_COMPLETE);
 
         rv = upstream->on_downstream_body_complete(downstream);
 
         if (rv != 0) {
-          downstream->set_response_state(Downstream::MSG_RESET);
+          downstream->set_response_state(DownstreamState::MSG_RESET);
         }
       }
     }
@@ -1277,15 +1279,16 @@ int on_frame_recv_callback(nghttp2_session *session, const nghttp2_frame *frame,
     if (frame->hd.flags & NGHTTP2_FLAG_END_STREAM) {
       downstream->disable_downstream_rtimer();
 
-      if (downstream->get_response_state() == Downstream::HEADER_COMPLETE) {
-        downstream->set_response_state(Downstream::MSG_COMPLETE);
+      if (downstream->get_response_state() ==
+          DownstreamState::HEADER_COMPLETE) {
+        downstream->set_response_state(DownstreamState::MSG_COMPLETE);
 
         auto upstream = downstream->get_upstream();
 
         rv = upstream->on_downstream_body_complete(downstream);
 
         if (rv != 0) {
-          downstream->set_response_state(Downstream::MSG_RESET);
+          downstream->set_response_state(DownstreamState::MSG_RESET);
         }
       }
     } else {
@@ -1445,7 +1448,7 @@ int on_data_chunk_recv_callback(nghttp2_session *session, uint8_t flags,
       return NGHTTP2_ERR_CALLBACK_FAILURE;
     }
 
-    downstream->set_response_state(Downstream::MSG_RESET);
+    downstream->set_response_state(DownstreamState::MSG_RESET);
   }
 
   call_downstream_readcb(http2session, downstream);
@@ -1539,8 +1542,8 @@ int on_frame_not_send_callback(nghttp2_session *session,
     return 0;
   }
 
-  // To avoid stream hanging around, flag Downstream::MSG_RESET.
-  downstream->set_response_state(Downstream::MSG_RESET);
+  // To avoid stream hanging around, flag DownstreamState::MSG_RESET.
+  downstream->set_response_state(DownstreamState::MSG_RESET);
   call_downstream_readcb(http2session, downstream);
 
   return 0;
@@ -1652,7 +1655,7 @@ nghttp2_session_callbacks *create_http2_downstream_callbacks() {
 int Http2Session::connection_made() {
   int rv;
 
-  state_ = Http2Session::CONNECTED;
+  state_ = Http2SessionState::CONNECTED;
 
   on_write_ = &Http2Session::downstream_write;
   on_read_ = &Http2Session::downstream_read;
@@ -1800,7 +1803,7 @@ int Http2Session::downstream_write() {
 
 void Http2Session::signal_write() {
   switch (state_) {
-  case Http2Session::DISCONNECTED:
+  case Http2SessionState::DISCONNECTED:
     if (!ev_is_active(&initiate_connection_timer_)) {
       if (LOG_ENABLED(INFO)) {
         LOG(INFO) << "Start connecting to backend server";
@@ -1812,9 +1815,11 @@ void Http2Session::signal_write() {
       ev_feed_event(conn_.loop, &initiate_connection_timer_, 0);
     }
     break;
-  case Http2Session::CONNECTED:
+  case Http2SessionState::CONNECTED:
     conn_.wlimit.startw();
     break;
+  default:
+    break;
   }
 }
 
@@ -1824,9 +1829,9 @@ struct ev_loop *Http2Session::get_loop() const {
 
 ev_io *Http2Session::get_wev() { return &conn_.wev; }
 
-int Http2Session::get_state() const { return state_; }
+Http2SessionState Http2Session::get_state() const { return state_; }
 
-void Http2Session::set_state(int state) { state_ = state; }
+void Http2Session::set_state(Http2SessionState state) { state_ = state; }
 
 int Http2Session::terminate_session(uint32_t error_code) {
   int rv;
@@ -1860,17 +1865,17 @@ int Http2Session::consume(int32_t stream_id, size_t len) {
 
 bool Http2Session::can_push_request(const Downstream *downstream) const {
   auto &req = downstream->request();
-  return state_ == CONNECTED &&
-         connection_check_state_ == CONNECTION_CHECK_NONE &&
-         (!req.connect_proto || settings_recved_);
+  return state_ == Http2SessionState::CONNECTED &&
+         connection_check_state_ == ConnectionCheck::NONE &&
+         (req.connect_proto == ConnectProto::NONE || settings_recved_);
 }
 
 void Http2Session::start_checking_connection() {
-  if (state_ != CONNECTED ||
-      connection_check_state_ != CONNECTION_CHECK_REQUIRED) {
+  if (state_ != Http2SessionState::CONNECTED ||
+      connection_check_state_ != ConnectionCheck::REQUIRED) {
     return;
   }
-  connection_check_state_ = CONNECTION_CHECK_STARTED;
+  connection_check_state_ = ConnectionCheck::STARTED;
 
   SSLOG(INFO, this) << "Start checking connection";
   // If connection is down, we may get error when writing data.  Issue
@@ -1889,7 +1894,7 @@ void Http2Session::reset_connection_check_timer(ev_tstamp t) {
 }
 
 void Http2Session::reset_connection_check_timer_if_not_checking() {
-  if (connection_check_state_ != CONNECTION_CHECK_NONE) {
+  if (connection_check_state_ != ConnectionCheck::NONE) {
     return;
   }
 
@@ -1899,7 +1904,7 @@ void Http2Session::reset_connection_check_timer_if_not_checking() {
 void Http2Session::connection_alive() {
   reset_connection_check_timer(CONNCHK_TIMEOUT);
 
-  if (connection_check_state_ == CONNECTION_CHECK_NONE) {
+  if (connection_check_state_ == ConnectionCheck::NONE) {
     return;
   }
 
@@ -1907,7 +1912,7 @@ void Http2Session::connection_alive() {
     SSLOG(INFO, this) << "Connection alive";
   }
 
-  connection_check_state_ = CONNECTION_CHECK_NONE;
+  connection_check_state_ = ConnectionCheck::NONE;
 
   submit_pending_requests();
 }
@@ -1922,7 +1927,7 @@ void Http2Session::submit_pending_requests() {
     }
 
     auto &req = downstream->request();
-    if (req.connect_proto && !settings_recved_) {
+    if (req.connect_proto != ConnectProto::NONE && !settings_recved_) {
       continue;
     }
 
@@ -1942,11 +1947,11 @@ void Http2Session::submit_pending_requests() {
   }
 }
 
-void Http2Session::set_connection_check_state(int state) {
+void Http2Session::set_connection_check_state(ConnectionCheck state) {
   connection_check_state_ = state;
 }
 
-int Http2Session::get_connection_check_state() const {
+ConnectionCheck Http2Session::get_connection_check_state() const {
   return connection_check_state_;
 }
 
@@ -1982,7 +1987,7 @@ int Http2Session::connected() {
   read_ = &Http2Session::read_clear;
   write_ = &Http2Session::write_clear;
 
-  if (state_ == PROXY_CONNECTING) {
+  if (state_ == Http2SessionState::PROXY_CONNECTING) {
     return do_write();
   }
 
@@ -1994,7 +1999,7 @@ int Http2Session::connected() {
   }
 
   if (connection_made() != 0) {
-    state_ = CONNECT_FAILING;
+    state_ = Http2SessionState::CONNECT_FAILING;
     return -1;
   }
 
@@ -2096,7 +2101,7 @@ int Http2Session::tls_handshake() {
   write_ = &Http2Session::write_tls;
 
   if (connection_made() != 0) {
-    state_ = CONNECT_FAILING;
+    state_ = Http2SessionState::CONNECT_FAILING;
     return -1;
   }
 
@@ -2183,10 +2188,10 @@ int Http2Session::write_void() {
 
 bool Http2Session::should_hard_fail() const {
   switch (state_) {
-  case PROXY_CONNECTING:
-  case PROXY_FAILED:
+  case Http2SessionState::PROXY_CONNECTING:
+  case Http2SessionState::PROXY_FAILED:
     return true;
-  case DISCONNECTED: {
+  case Http2SessionState::DISCONNECTED: {
     const auto &proxy = get_config()->downstream_http_proxy;
     return !proxy.host.empty();
   }
@@ -2215,7 +2220,7 @@ int Http2Session::handle_downstream_push_promise(Downstream *downstream,
 
   auto handler = upstream->get_client_handler();
 
-  auto promised_dconn = make_unique<Http2DownstreamConnection>(this);
+  auto promised_dconn = std::make_unique<Http2DownstreamConnection>(this);
   promised_dconn->set_client_handler(handler);
 
   auto ptr = promised_dconn.get();
@@ -2225,7 +2230,7 @@ int Http2Session::handle_downstream_push_promise(Downstream *downstream,
     return -1;
   }
 
-  auto promised_sd = make_unique<StreamData>();
+  auto promised_sd = std::make_unique<StreamData>();
 
   nghttp2_session_set_stream_user_data(session_, promised_stream_id,
                                        promised_sd.get());
@@ -2282,7 +2287,7 @@ int Http2Session::handle_downstream_push_promise_complete(
 
   auto upstream = promised_downstream->get_upstream();
 
-  promised_downstream->set_request_state(Downstream::MSG_COMPLETE);
+  promised_downstream->set_request_state(DownstreamState::MSG_COMPLETE);
   promised_downstream->set_request_header_sent(true);
 
   if (upstream->on_downstream_push_promise_complete(downstream,
@@ -2314,7 +2319,7 @@ Http2Session::get_downstream_addr_group() const {
 }
 
 void Http2Session::add_to_avail_freelist() {
-  if (freelist_zone_ != FREELIST_ZONE_NONE) {
+  if (freelist_zone_ != FreelistZone::NONE) {
     return;
   }
 
@@ -2324,13 +2329,13 @@ void Http2Session::add_to_avail_freelist() {
                       << group_->shared_addr->http2_avail_freelist.size();
   }
 
-  freelist_zone_ = FREELIST_ZONE_AVAIL;
+  freelist_zone_ = FreelistZone::AVAIL;
   group_->shared_addr->http2_avail_freelist.append(this);
   addr_->in_avail = true;
 }
 
 void Http2Session::add_to_extra_freelist() {
-  if (freelist_zone_ != FREELIST_ZONE_NONE) {
+  if (freelist_zone_ != FreelistZone::NONE) {
     return;
   }
 
@@ -2340,15 +2345,15 @@ void Http2Session::add_to_extra_freelist() {
                       << addr_->http2_extra_freelist.size();
   }
 
-  freelist_zone_ = FREELIST_ZONE_EXTRA;
+  freelist_zone_ = FreelistZone::EXTRA;
   addr_->http2_extra_freelist.append(this);
 }
 
 void Http2Session::remove_from_freelist() {
   switch (freelist_zone_) {
-  case FREELIST_ZONE_NONE:
+  case FreelistZone::NONE:
     return;
-  case FREELIST_ZONE_AVAIL:
+  case FreelistZone::AVAIL:
     if (LOG_ENABLED(INFO)) {
       SSLOG(INFO, this) << "Remove from http2_avail_freelist, group=" << group_
                         << ", freelist.size="
@@ -2357,7 +2362,7 @@ void Http2Session::remove_from_freelist() {
     group_->shared_addr->http2_avail_freelist.remove(this);
     addr_->in_avail = false;
     break;
-  case FREELIST_ZONE_EXTRA:
+  case FreelistZone::EXTRA:
     if (LOG_ENABLED(INFO)) {
       SSLOG(INFO, this) << "Remove from http2_extra_freelist, addr=" << addr_
                         << ", freelist.size="
@@ -2365,34 +2370,35 @@ void Http2Session::remove_from_freelist() {
     }
     addr_->http2_extra_freelist.remove(this);
     break;
-  case FREELIST_ZONE_GONE:
+  case FreelistZone::GONE:
     return;
   }
 
-  freelist_zone_ = FREELIST_ZONE_NONE;
+  freelist_zone_ = FreelistZone::NONE;
 }
 
 void Http2Session::exclude_from_scheduling() {
   remove_from_freelist();
-  freelist_zone_ = FREELIST_ZONE_GONE;
+  freelist_zone_ = FreelistZone::GONE;
 }
 
 DefaultMemchunks *Http2Session::get_request_buf() { return &wb_; }
 
 void Http2Session::on_timeout() {
   switch (state_) {
-  case PROXY_CONNECTING: {
+  case Http2SessionState::PROXY_CONNECTING: {
     auto worker_blocker = worker_->get_connect_blocker();
     worker_blocker->on_failure();
     break;
   }
-  case CONNECTING: {
+  case Http2SessionState::CONNECTING:
     SSLOG(WARN, this) << "Connect time out; addr="
                       << util::to_numeric_addr(raddr_);
 
     downstream_failure(addr_, raddr_);
     break;
-  }
+  default:
+    break;
   }
 }
 
